@@ -690,8 +690,6 @@ class Address(object):
     def dataTransaction(self, data, timestamp=0, baseFee=pywaves.DEFAULT_BASE_FEE, minimalFee=500000):
         dataTransaction = transaction_pb2.DataTransactionData()
 
-        print(data)
-
         dataList = []
         for d in data:
             entry = transaction_pb2.DataTransactionData.DataEntry()
@@ -708,7 +706,6 @@ class Address(object):
                 listEntry['key'] = d['key']
                 listEntry['value'] = d['value']
                 listEntry['type'] = 'string'
-                print(listEntry)
             elif d['type'] == 'integer':
                 entry.key = d['key']
                 entry.int_value = d['value']
@@ -798,12 +795,13 @@ class Address(object):
 
         return req
 
-    def _postOrder(self, amountAsset, priceAsset, orderType, amount, price, maxLifetime=30 * 86400, matcherFee=pywaves.DEFAULT_MATCHER_FEE, timestamp=0):
+    def _postOrder(self, amountAsset, priceAsset, orderType, amount, price, maxLifetime=30 * 86400, matcherFee=pywaves.DEFAULT_MATCHER_FEE, timestamp=0, matcherFeeAssetId=''):
         if timestamp == 0:
             timestamp = int(time.time() * 1000)
         expiration = timestamp + maxLifetime * 1000
         asset1 = b'\0' if amountAsset.assetId == '' else b'\1' + base58.b58decode(amountAsset.assetId)
         asset2 = b'\0' if priceAsset.assetId == '' else b'\1' + base58.b58decode(priceAsset.assetId)
+        matcherFeeAssetIdPart = b'\0' if matcherFeeAssetId == '' else b'\1' + base58.b58decode(matcherFeeAssetId)
         sData = b'\3' + \
                 base58.b58decode(self.publicKey) + \
                 base58.b58decode(self.pywaves.MATCHER_PUBLICKEY) + \
@@ -815,7 +813,7 @@ class Address(object):
                 struct.pack(">Q", timestamp) + \
                 struct.pack(">Q", expiration) + \
                 struct.pack(">Q", matcherFee) + \
-                b'\0'
+                matcherFeeAssetIdPart
         signature = crypto.sign(self.privateKey, sData)
         otype = "buy" if orderType == b'\0' else "sell"
         data = json.dumps({
@@ -832,7 +830,8 @@ class Address(object):
             "expiration": expiration,
             "matcherFee": matcherFee,
             "signature": signature,
-            "version": 3
+            "version": 3,
+            "matcherFeeAssetId": matcherFeeAssetId
         })
         req = self.pywaves.wrapper('/matcher/orderbook', data, host=self.pywaves.MATCHER)
         id = -1
@@ -906,22 +905,22 @@ class Address(object):
             return id
 
     def buy(self, assetPair, amount, price, maxLifetime=30 * 86400, matcherFee=pywaves.DEFAULT_MATCHER_FEE,
-            timestamp=0):
+            timestamp=0, matcherFeeAssetId=''):
         assetPair.refresh()
         #normPrice = int(pow(10, 8 + assetPair.asset2.decimals - assetPair.asset1.decimals) * price)
         normPrice = int(round(pow(10, 8 + assetPair.asset2.decimals - assetPair.asset1.decimals) * price))
-        id = self._postOrder(assetPair.asset1, assetPair.asset2, b'\0', amount, normPrice, maxLifetime, matcherFee, timestamp)
+        id = self._postOrder(assetPair.asset1, assetPair.asset2, b'\0', amount, normPrice, maxLifetime, matcherFee, timestamp, matcherFeeAssetId)
         if self.pywaves.OFFLINE:
             return id
         elif id != -1:
             return self.pywaves.Order(id, assetPair, self)
 
     def sell(self, assetPair, amount, price, maxLifetime=30 * 86400, matcherFee=pywaves.DEFAULT_MATCHER_FEE,
-             timestamp=0):
+             timestamp=0, matcherFeeAssetId=''):
         assetPair.refresh()
         #normPrice = int(pow(10, 8 + assetPair.asset2.decimals - assetPair.asset1.decimals) * price)
         normPrice = int(round(pow(10, 8 + assetPair.asset2.decimals - assetPair.asset1.decimals) * price))
-        id = self._postOrder(assetPair.asset1, assetPair.asset2, b'\1', amount, normPrice, maxLifetime, matcherFee, timestamp)
+        id = self._postOrder(assetPair.asset1, assetPair.asset2, b'\1', amount, normPrice, maxLifetime, matcherFee, timestamp, matcherFeeAssetId)
         if self.pywaves.OFFLINE:
             return id
         elif id != -1:
@@ -1299,7 +1298,7 @@ class Address(object):
             parameterBytes = b''
             for param in params:
                 if param['type'] == 'integer':
-                    parameterBytes += b'\0' + struct.pack(">Q", param['value'])
+                    parameterBytes += b'\0' + struct.pack(">q", param['value'])
                 elif param['type'] == 'binary':
                     parameterBytes += b'\1' + struct.pack(">I", len(param['value'])) + crypto.str2bytes(param['value'])
                 elif param['type'] == 'string':
@@ -1311,6 +1310,25 @@ class Address(object):
                         parameterBytes += b'\6'
                     else:
                         parameterBytes += b'\7'
+                elif param['type'] == 'list':
+                    parameterBytes += b'\x0b'
+                    parameterBytes += struct.pack(">I", len(param['value']))
+                    for nestedParam in param['value']:
+                        if nestedParam['type'] == 'integer':
+                            parameterBytes += b'\0' + struct.pack(">Q", nestedParam['value'])
+                        elif nestedParam['type'] == 'binary':
+                            parameterBytes += b'\1' + struct.pack(">I", len(nestedParam['value'])) + crypto.str2bytes(
+                                nestedParam['value'])
+                        elif nestedParam['type'] == 'string':
+                            parameterBytes += b'\2' + struct.pack(">I",
+                                                                  len(crypto.str2bytes(
+                                                                      nestedParam['value']))) + crypto.str2bytes(
+                                nestedParam['value'])
+                        elif nestedParam['type'] == 'boolean':
+                            if nestedParam['value'] == True:
+                                parameterBytes += b'\6'
+                            else:
+                                parameterBytes += b'\7'
             paymentBytes = b''
             for payment in payments:
                 currentPaymentBytes = b''
